<?php
/**
 * SourceBans default controller
 * 
 * @author GameConnect
 * @copyright (C)2007-2013 GameConnect.net.  All rights reserved.
 * @link http://www.sourcebans.net
 * 
 * @package sourcebans.controllers
 * @since 2.0
 */
class DefaultController extends Controller
{
	/**
	 * Declares class-based actions.
	 */
	public function actions()
	{
		return array(
			// captcha action renders the CAPTCHA image displayed on the contact page
			'captcha'=>array(
				'class'=>'CCaptchaAction',
				'backColor'=>0xFFFFFF,
			),
			// page action renders "static" pages stored under 'application/views/default/pages'
			// They can be accessed via: index.php?r=default/page&view=FileName
			'page'=>array(
				'class'=>'CViewAction',
			),
		);
	}

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
			'accessControl', // perform access control for CRUD operations
		);
	}

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
			array('deny',  // deny anonymous users
				'actions'=>array('account'),
				'users'=>array('?'),
			),
			array('deny',
				'actions'=>array('appeal'),
				'expression'=>'!SourceBans::app()->settings->enable_appeals',
			),
			array('deny',
				'actions'=>array('report'),
				'expression'=>'!SourceBans::app()->settings->enable_reports',
			),
			array('allow', // allow all users
				'users'=>array('*'),
			),
		);
	}

	/**
	 * This is the default 'index' action that is invoked
	 * when an action is not explicitly requested by users.
	 */
	public function actionIndex()
	{
		$defaultAction = 'action' . ucfirst(SourceBans::app()->settings->default_page);
		
		$this->$defaultAction();
	}

	/**
	 * Displays the dashboard page
	 */
	public function actionDashboard()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.dashboard.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.dashboard.title'),
		);
		
		$bans = SBBan::model()->search();
		$bans->criteria->limit = 10;
		$bans->criteria->with = array('server', 'server.game');
		$bans->pagination = false;
		
		$blocks = SBBlock::model()->search();
		$blocks->criteria->limit = 10;
		$blocks->criteria->with = 'ban';
		$blocks->pagination = false;
		
		$servers = SBServer::model()->search();
		$servers->criteria->scopes = 'enabled';
		$servers->pagination = false;
		
		$this->render('dashboard', array(
			'bans' => $bans,
			'blocks' => $blocks,
			'servers' => $servers,
			'total_bans' => SBBan::model()->count(),
			'total_blocks' => SBBlock::model()->count(),
		));
	}

	/**
	 * Displays the bans page
	 */
	public function actionBans()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.bans.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.bans.title'),
		);
		
		$hideInactive = Yii::app()->request->getQuery('hideinactive', 'false') == 'true';
		$search = Yii::app()->request->getQuery('q');
		
		$ban = new SBBan;
		
		$bans = new SBBan('search');
		$bans->unsetAttributes();  // clear any default values
		if(isset($_GET['SBBan']))
			$bans->attributes=$_GET['SBBan'];
		
		$comment = new SBComment;
		$comment->object_type = SBComment::TYPE_BAN;
		
		$this->render('bans', array(
			'ban' => $ban,
			'bans' => $bans,
			'comment' => $comment,
			'hideInactive' => $hideInactive,
			'search' => $search,
			'total_bans' => SBBan::model()->count(array(
				'scopes' => $hideInactive ? 'active' : null,
			)),
		));
	}

	/**
	 * Displays the servers page
	 */
	public function actionServers()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.servers.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.servers.title'),
		);
		
		$servers = SBServer::model()->search();
		$servers->criteria->scopes = 'enabled';
		$servers->pagination = false;
		
		$this->render('servers', array(
			'servers' => $servers,
		));
	}

	/**
	 * Displays the report player page
	 */
	public function actionReport()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.report.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.report.title'),
		);
		
		$model = new SBReport;
		$model->steam = 'STEAM_';
		
		// if it is ajax validation request
		if(isset($_POST['ajax']) && $_POST['ajax']==='report-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
		
		if(isset($_POST['SBReport']))
		{
			$model->attributes=$_POST['SBReport'];
			if($model->save())
			{
				SourceBans::log('Report added', 'Player "' . $model->name . '" was reported');
				Yii::app()->user->setFlash('success', Yii::t('sourcebans', 'Saved successfully'));
				
				$this->refresh();
			}
		}
		
		$games = SBGame::model()->with('servers:enabled')->findAll(array(
			'condition' => 'servers.id IS NOT NULL',
			'order' => 'name, servers.host, servers.port',
		));
		
		$this->render('report', array(
			'model' => $model,
			'games' => $games,
		));
	}

	/**
	 * Displays the appeal ban page
	 */
	public function actionAppeal()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.appeal.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.appeal.title'),
		);
		
		$model = new SBAppeal;
		$model->ban_steam = 'STEAM_';
		$model->ban_ip = Yii::app()->request->userHostAddress;
		
		// if it is ajax validation request
		if(isset($_POST['ajax']) && $_POST['ajax']==='appeal-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
		
		if(isset($_POST['SBAppeal']))
		{
			$model->attributes=$_POST['SBAppeal'];
			if($model->save())
			{
				SourceBans::log('Appeal added', 'Ban against "' . $model->user_email . '" was appealed');
				Yii::app()->user->setFlash('success', Yii::t('sourcebans', 'Saved successfully'));
				
				$this->refresh();
			}
		}
		
		$this->render('appeal', array(
			'model' => $model,
		));
	}

	/**
	 * Displays the account page
	 */
	public function actionAccount()
	{
		$this->layout='//layouts/column2';
		
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.account.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.account.title'),
		);
		
		$this->menu=array(
			array('label'=>Yii::t('sourcebans', 'controllers.default.account.menu.permissions'), 'url'=>'#permissions'),
			array('label'=>Yii::t('sourcebans', 'controllers.default.account.menu.settings'), 'url'=>'#settings'),
			array('label'=>Yii::t('sourcebans', 'controllers.default.account.menu.email'), 'url'=>'#email'),
			array('label'=>Yii::t('sourcebans', 'controllers.default.account.menu.password'), 'url'=>'#password'),
			array('label'=>Yii::t('sourcebans', 'controllers.default.account.menu.server-password'), 'url'=>'#server-password'),
		);
		
		$model=new AccountForm;

		// if it is ajax validation request
		if(isset($_POST['ajax']) && in_array($_POST['ajax'],array(
			'email-form',
			'password-form',
			'server-password-form',
			'settings-form',
		)))
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}

		// collect user input data
		if(isset($_POST['AccountForm']))
		{
			$model->setScenario($_POST['scenario']);
			$model->attributes=$_POST['AccountForm'];
			// validate user input and redirect to the previous page if valid
			if($model->validate() && $model->save())
				$this->redirect(array('','#'=>'permissions'));
		}

		// display the account form
		$this->render('account', array(
			'model' => $model,
		));
	}

	/**
	 * This is the action to handle external exceptions.
	 */
	public function actionError()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.error.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.error.title'),
		);
		
		if($error=Yii::app()->errorHandler->error)
		{
			if(Yii::app()->request->isAjaxRequest)
				echo $error['message'];
			else
				$this->render('error', $error);
		}
	}

	/**
	 * Displays the login page
	 */
	public function actionLogin()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.login.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.login.title'),
		);
		
		$model=new LoginForm;

		// if it is ajax validation request
		if(isset($_POST['ajax']) && $_POST['ajax']==='login-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}

		// collect user input data
		if(isset($_POST['LoginForm']))
		{
			$model->attributes=$_POST['LoginForm'];
			// validate user input and redirect to the previous page if valid
			if($model->validate() && $model->login())
				$this->redirect(Yii::app()->user->returnUrl);
		}

		// display the login form
		$this->render('login', array(
			'model' => $model,
		));
	}

	/**
	 * Logs out the current user and redirect to homepage.
	 */
	public function actionLogout()
	{
		Yii::app()->user->logout();
		$this->redirect(Yii::app()->homeUrl);
	}

	/**
	 * Displays the lost password page
	 */
	public function actionLostPassword()
	{
		$this->pageTitle=Yii::t('sourcebans', 'controllers.default.lostPassword.title');
		
		$this->breadcrumbs=array(
			Yii::t('sourcebans', 'controllers.default.lostPassword.title'),
		);
		
		$model=new LostPasswordForm;

		// if it is ajax validation request
		if(isset($_POST['ajax']) && $_POST['ajax']==='lost-password-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}

		// collect user input data
		if(isset($_POST['LostPasswordForm']))
		{
			$model->attributes=$_POST['LostPasswordForm'];
			// validate user input and redirect to the previous page if valid
			if($model->validate() && $model->reset())
				$this->redirect(Yii::app()->user->returnUrl);
		}

		$email = Yii::app()->request->getQuery('email');
		$validationKey = Yii::app()->request->getQuery('key');
		if(!empty($email) && !empty($validationKey))
		{
			/** @var SBAdmin $admin */
			$admin = SBAdmin::model()->findByAttributes(array(
				'email' => $email,
				'validation_key' => $validationKey,
			));
			if($admin === null)
				throw new CHttpException(403, 'The validation key does not match the email address for this reset request.');
			
			$password = Yii::app()->securityManager->generateRandomString(8, false);
			Yii::app()->mailer->AddAddress($admin->email);
			Yii::app()->mailer->Subject = Yii::t('sourcebans', 'controllers.default.lostPassword.subject');
			Yii::app()->mailer->MsgHtml(Yii::t('sourcebans', 'controllers.default.lostPassword.body', array(
				'{name}' => $admin->name,
				'{password}' => $password,
				'{link}' => CHtml::link(Yii::t('sourcebans', 'controllers.default.account.title'), array('default/account')),
			)));
			if(!Yii::app()->mailer->Send())
				throw new CHttpException(500, 'Please try again later or contact your system administrator.');
			
			$admin->new_password = $password;
			$admin->validation_key = null;
			$admin->save(false);
			$this->redirect(Yii::app()->user->homeUrl);
		}

		// display the lost password form
		$this->render('lostpassword', array(
			'model' => $model,
		));
	}
}